
#include "../melee.h"
#include "../ai.h"
/////////////////// INITIALIZING AI /////////////////////////////////
AI_Player *init_ai(Ship *ai, char *id)
{
  if (!strcmp(id, "arisk")) return new AI_Player_Arilou(ai);
  if (!strcmp(id, "chmav")) return new AI_Player_Chmmr(ai);
  if (!strcmp(id, "earcr")) return new AI_Player_Earthling(ai);
//  if (!strcmp(id, "ilwav")) return new AI_Player_Ilwrath(ai);
  if (!strcmp(id, "pkufu")) return new AI_Player_Pkunk(ai);
  if (!strcmp(id, "yehte")) return new AI_Player_Yehat(ai);
  if (!strcmp(id, "utwju")) return new AI_Player_Utwig(ai);
  if (!strcmp(id, "slypr")) return new AI_Player_Slylandro(ai);
//  if (!strcmp(id, "supbl")) return new AI_Player_Supox(ai);

  return NULL;
}
////////////////////// BASE AI CLASS //////////////////////////////
AI_Player::AI_Player(Ship *ai)
{
      ai_ship = ai;
 select_state = true;
}
////////////// tween of trajectrory but differ args /////////////////////////
double AI_Player::trajectory_angle(double x1, double y1, double x2, double y2)
{ 
  double dx = min_delta(x1, x2, X_MAX);
  double dy = min_delta(y1, y2, Y_MAX);
  double alpha = atan(dy / dx);

  if(dx < 0.0)
    alpha += PI;
  alpha = normalize(alpha, PI * 2.0);

  alpha = ((alpha + PI) / (PI * 2.0)) * 360.0;
  alpha = normalize(alpha, 360.0);
 return alpha;
}
/////////////////////// Slowdown ship ////////////////////////////////////
///////////////////// Seems work ideal ;) ////////////////////////////////
void AI_Player::Stop()
{
 k.turn_left  = false;
 k.turn_right = false;
 k.thrust     = false;

 double     ai_x = ai_ship->normal_x();
 double     ai_y = ai_ship->normal_y();
 double    ai_vx = ai_ship->get_vx(); 
 double    ai_vy = ai_ship->get_vy(); 

 if (sqrt(ai_vx*ai_vx+ai_vy*ai_vy)<0.10)  // speed is very low = stop
  {
   select_state = true; //FINISH 'STOP'
   return;
  }
 double alpha = trajectory_angle(ai_x, ai_y, ai_x+ai_vx, ai_y+ai_vy); // get angle of direction
        alpha = ai_ship->get_angle() - alpha;

  if (alpha < 0.0) alpha += 360.0;

  if (alpha < 180.0 - 5.6251) 
   k.turn_right = true; 
  else
  if (alpha > 180.0 + 5.6251) 
   k.turn_left  = true;
  else
   k.thrust = true;
}
//////////////////////// Chase enemy ////////////////////////////////////
///////////////////// Seems work good ;) ////////////////////////////////
void AI_Player::Chase(SpaceLocation *enemy)
{
 k.turn_left    = false;
 k.turn_right   = false;
 k.thrust       = true;

 double  a_x = ai_ship->normal_x(); 
 double  a_y = ai_ship->normal_y();
 double a_vx = ai_ship->get_vx(); 
 double a_vy = ai_ship->get_vy();
 double  e_x = enemy->normal_x(); 
 double  e_y = enemy->normal_y();
 double e_vx = enemy->get_vx();
 double e_vy = enemy->get_vy();

// double alpha = ai_ship->get_angle() - enemy->trajectory_angle(ai_ship); 
 double alpha = ai_ship->get_angle() - trajectory_angle(e_x + e_vx, e_y + e_vy, 
                                                        a_x + a_vx, a_y + a_vy);

 if (alpha < 0.0) alpha += 360.0;

 if (alpha < 180.0 - 5.6251) 
   k.turn_right = true;
 else
  if (alpha > 180.0 + 5.6251) 
   k.turn_left  = true;
 else
   select_state=true;
}
///////////////////// Runaway from enemy ///////////////////////////
///////////////////// Seems work good ;) ////////////////////////////////
void AI_Player::Runaway(SpaceLocation *enemy, double range)
{
 k.turn_left  = false;
 k.turn_right = false;
 k.thrust     = true;

 double    ai_angle = ai_ship->get_angle(); 
 double enemy_angle = enemy->trajectory_angle(ai_ship);

 if (ai_angle < enemy_angle) k.turn_right = true;
 if (ai_angle > enemy_angle) k.turn_left  = true;

 if (ai_ship->distance(enemy)>range) 
  select_state=true;
}
//////////////// Turn face to enemy for 'angle' /////////////////////////
///////////////////// Seems work good ;) ////////////////////////////////
void AI_Player::Turnface(SpaceLocation *enemy, double angle)
{
 k.turn_left    = false;
 k.turn_right   = false;
 double alpha = ai_ship->get_angle() - enemy->trajectory_angle(ai_ship); 
 if (alpha < 0.0) alpha += 360.0;

 if (alpha < 180.0 - 5.6251 - angle) 
  k.turn_right = true; 
 else
  if (alpha > 180.0 + 5.6251 + angle) 
   k.turn_left  = true;
 else
   select_state=true;
}
/////////////////////////  ATTACK   ////////////////////////////////////
void AI_Player::Attack(SpaceLocation *enemy)
{
 k.turn_left  = false;
 k.turn_right = false;

 double  a_x = ai_ship->normal_x(); 
 double  a_y = ai_ship->normal_y();
 double a_vx = ai_ship->get_vx(); 
 double a_vy = ai_ship->get_vy();
 double  e_x = enemy->normal_x(); 
 double  e_y = enemy->normal_y();
 double e_vx = enemy->get_vx();
 double e_vy = enemy->get_vy();

 // targeting for future position is still not perfect :((((
// double alpha = ai_ship->get_angle() - trajectory_angle(e_x + e_vx, e_y + e_vy, 
//                                                        a_x + a_vx, a_y + a_vy);

 double alpha = ai_ship->get_angle() - trajectory_angle(e_x + e_vx, e_y + e_vy, 
                                                        a_x + a_vx, a_y + a_vy);

 if (alpha < 0.0) alpha += 360.0;

 if (alpha < 180.0 - 5.6251) 
  k.turn_right = true; 
 else
  if (alpha > 180.0 + 5.6251) 
   k.turn_left  = true;
 else
  { 
    select_state=true;
    k.fire_weapon = true;
  }
}  
/////////////// ENEMY SHOT IS TARGETED TO US? //////////////////
////////////// Seems work good. But not ideal ;) ///////////////
bool AI_Player::isTarget(SpaceLocation *enemy)
{
  double  ai_x = ai_ship->normal_x();
  double  ai_y = ai_ship->normal_y();
  double ai_vx = ai_ship->get_vx(); 
  double ai_vy = ai_ship->get_vy(); 

  double  enemy_x = enemy->normal_x();
  double  enemy_y = enemy->normal_y();
  double enemy_vx = enemy->get_vx(); 
  double enemy_vy = enemy->get_vy(); 

  double      range = ai_ship->distance(enemy);
  double next_range = distance_from(ai_x + ai_vx, ai_y + ai_vy, 
                       enemy_x + enemy_vx, enemy_y + enemy_vy);

 if (next_range >= range) return false; //differ directions 

 double range_step = range - next_range;
 
    double k = range / range_step;
  next_range = distance_from(ai_x + ai_vx*k, ai_y + ai_vy*k, 
                enemy_x + enemy_vx*k, enemy_y + enemy_vy*k);

  if (next_range<=50.0) // in ideal...there must be 'ship & shot width' factor  
   return true;         // but not only range between 2 points!!!!
  else
   return false;
}

////////////////// Trying to EVADE from enemy shot ///////////
void AI_Player::Evade(SpaceLocation *enemy)
{

 select_state = true;
 if (!isTarget(enemy)) return; // We're targeted?

 k.turn_left  = false;
 k.turn_right = false;
 k.thrust     = true;

 double x1 = ai_ship->normal_x(); 
 double y1 = ai_ship->normal_y(); 
 double x2 = enemy->normal_x();
 double y2 = enemy->normal_y();

 bool left=false, up=false; 

 if (x1 <= x2) left = true;
 if (y1 <= y2)   up = true;

 if (left && up) 
  k.turn_right = true; 
 else
  k.turn_left  = true;

  select_state=true;
}
bool AI_Player::nearPlanet()
{
 if (ai_ship->distance(spacePlanet)<500.0) 
  return true;
 else
 return false;
}
////////////////// simple check all enemy shots for 'danger' //////////
void AI_Player::CheckShots()
{
 SpaceObject *shot;
 shot = spaceFrame->getFirstObject(LAYER_SHOTS);
 double shot_range;

 while (shot!=NULL)
 {
  if (!ai_ship->isAlly(shot)) {
  shot_range = ai_ship->distance(shot); 
   ProcessShot(shot, shot_range);          //process this shot
  }
  shot=shot->next;
 }
}
/////////////////  what we must do with 'danger shot' ///////////
void AI_Player::ProcessShot(SpaceLocation *shot, double range)
{
}

void AI_Player::calculate()
{
 k.turn_left    = false;
 k.turn_right   = false;
 k.fire_weapon  = false;
 k.fire_special = false;
 k.thrust       = false;
}
void AI_Player::change_state()
{
}
//////////
int KeyState::convert() {
  int a;
  a = 0;
  if (thrust)       a |= keyflag::thrust;
  if (turn_left)    a |= keyflag::left;
  if (turn_right)   a |= keyflag::right;
  if (fire_weapon)  a |= keyflag::fire;
  if (fire_special) a |= keyflag::special;
  return a;
}

