#include "../melee.h"
#include "../ai.h"


void Control::load(char *initfile, char *section) {
  set_config_file(initfile);
  strcpy(name, section);
  type       = abs( get_config_int(section, "AI", 0)) % 3;
  thrust   = get_config_int   (section, "Thrust", 0);
  left     = get_config_int   (section, "TurnLeft", 0);
  right    = get_config_int   (section, "TurnRight", 0);
  fire     = get_config_int   (section, "FireWeapon", 0);
  special  = get_config_int   (section, "FireSpecial", 0);
}
void Control::save(char *initfile, char *section) {
  set_config_file(initfile);

  set_config_int   (section, "AI", type);
  set_config_int   (section, "Thrust", thrust);
  set_config_int   (section, "TurnLeft", left);
  set_config_int   (section, "TurnRight", right);
  set_config_int   (section, "FireWeapon", fire);
  set_config_int   (section, "FireSpecial", special);

}

void Control::set_ship(Ship *s, char *n) {
  my_ship = s;
  switch (type) {
    case 2:
      if (ai) delete ai;
      ai = init_ai(s, n);
      break;
    default:
      break;
    }
  return;
}

void ControlKB::initialize(char* init_string) {
  strcpy(desc, init_string);
  loadKeys();
  ai = NULL;
  type       = abs( get_config_int(init_string, "AI", 0)) % 3;
}

void ControlKB::deinitialize() {
  desc[0] = '\0'; // Clear it, more or less;
  if (ai) delete ai;
  ai = NULL;
}

void ControlKB::loadKeys() {

  set_config_file("scp.ini");

  thrust  = get_config_int(desc, "Thrust", 0);
  left    = get_config_int(desc, "TurnLeft", 0);
  right   = get_config_int(desc, "TurnRight", 0);
  fire    = get_config_int(desc, "FireWeapon", 0);
  special = get_config_int(desc, "FireSpecial", 0);

}
int ControlKB::human () {
  int r;
  r = 0;
  if (key[thrust]) r |= keyflag::thrust;
  if (key[left]) r |= keyflag::left;
  if (key[right]) r |= keyflag::right;
  if (key[fire]) r |= keyflag::fire;
  if (key[special]) r |= keyflag::special;
  return r;
  }
int ControlKB::bot () {
  double a;
  int r = 0;
  if (!my_ship) return r;
  if (!my_ship->target) return r;
  a = my_ship->trajectory_angle(my_ship->target) - my_ship->get_angle();
  a = fmod(a + 360, 360);
  if (a < 180) {
    r |= keyflag::right;
    }
  else {
    r |= keyflag::left;
    return r;
    }
  a = int(my_ship->distance(my_ship->target));
  if (a > 2000) {
    r |= keyflag::thrust;
    }
  else {
    r |= keyflag::thrust;
    r |= keyflag::fire;
    }
  return r;
  }

static const int buflen = 4;
static int buffy[buflen];
static int pos = 0;

void ControlKB::act(int a) {
  int i;
  if (type) {
    if (random()&3) {
      buffy[pos] = a;
      pos = (pos + 1) % buflen;
      a = buffy[pos];
      }
    else a = buffy[(pos+1) % buflen];
    }
  i = my_ship->nextkeys;
  my_ship->nextkeys = a;
  my_ship->thrust       = 1&&(i & keyflag::thrust);
  my_ship->turn_left    = 1&&(i & keyflag::left);
  my_ship->turn_right   = 1&&(i & keyflag::right);
  my_ship->fire_weapon  = 1&&(i & keyflag::fire);
  my_ship->fire_special = 1&&(i & keyflag::special);
  return;
  }

void ControlKB::calculate() {
  unsigned int action = 0;
  switch (type) {
    case 2:
      if (ai) {
        ai->calculate();
        action = ai->k.convert();
        }
      else action = bot();
      break;
    case 1:
      action = bot();
      break;
    case 0:
      action = human();
      break;
    }
  act(action);
  return;
}

