
#include <allegro.h>


#include "../melee.h"
#include "../libs.h"

PPCOL_MASK *pix;

SpaceSprite::SpaceSprite(DATAFILE *sprites, int sprite_count)
{
  int i, w2, h2, w3, h3;
  BITMAP *bmp1, *bmp2, *bmp3;

  count = sprite_count;

  s = new RLE_SPRITE*[count * 3];
  m = new PPCOL_MASK*[count * 3];
	//MYCODE
  b = new BITMAP*[count];

  w = ((RLE_SPRITE *)sprites[0].dat)->w;
  h = ((RLE_SPRITE *)sprites[0].dat)->h;

  w2 = w / 2; if(w2 < 1) w2 = 1;
  h2 = h / 2; if(h2 < 1) h2 = 1;
  w3 = w / 4; if(w3 < 1) w3 = 1;
  h3 = h / 4; if(h3 < 1) h3 = 1;

  bmp2 = create_bitmap(w2, h2);
  bmp3 = create_bitmap(w3, h3);

  for(i = 0; i < count; i++) {

		bmp1 = create_bitmap(w, h);

    clear_to_color(bmp1, bitmap_mask_color(bmp1));
    clear_to_color(bmp2, bitmap_mask_color(bmp2));
    clear_to_color(bmp3, bitmap_mask_color(bmp3));

    tw_draw_rle_sprite(bmp1, (RLE_SPRITE *)(sprites[i].dat), 0, 0);
    aa_stretch_sprite(bmp2, bmp1, 0, 0, w2, h2);
    aa_stretch_sprite(bmp3, bmp1, 0, 0, w3, h3);

    s[i]                 = get_rle_sprite(bmp1);
    s[i + count]         = get_rle_sprite(bmp2);
    s[i + count + count] = get_rle_sprite(bmp3);

    m[i]                 = create_ppcol_mask(bmp1);
    m[i + count]         = create_ppcol_mask(bmp2);
    m[i + count + count] = create_ppcol_mask(bmp3);

		//MYCODE
		b[i]=bmp1;
  }

	
	//MYCOMMENT
  //destroy_bitmap(bmp1);
  destroy_bitmap(bmp2);
  destroy_bitmap(bmp3);
}

SpaceSprite::SpaceSprite(BITMAP **bmp, int bmp_count)
{
  int i, w2, h2, w3, h3;
  BITMAP *bmp1, *bmp2, *bmp3;

  count = bmp_count;

  s = new RLE_SPRITE*[count * 3];
  m = new PPCOL_MASK*[count * 3];
	//MYCODE
  b = new BITMAP*[count];

  w = bmp[0]->w;
  h = bmp[0]->h;

  w2 = w / 2; if(w2 < 1) w2 = 1;
  h2 = h / 2; if(h2 < 1) h2 = 1;
  w3 = w / 4; if(w3 < 1) w3 = 1;
  h3 = h / 4; if(h3 < 1) h3 = 1;

  bmp2 = create_bitmap(w2, h2);
  bmp3 = create_bitmap(w3, h3);

  for(i = 0; i < count; i++) {

		bmp1 = create_bitmap(w, h);

    clear_to_color(bmp1, bitmap_mask_color(bmp1));
    clear_to_color(bmp2, bitmap_mask_color(bmp2));
    clear_to_color(bmp3, bitmap_mask_color(bmp3));

    tw_draw_sprite(bmp1, bmp[i], 0, 0);
    aa_stretch_sprite(bmp2, bmp1, 0, 0, w2, h2);
    aa_stretch_sprite(bmp3, bmp1, 0, 0, w3, h3);

    s[i]                 = get_rle_sprite(bmp1);
    s[i + count]         = get_rle_sprite(bmp2);
    s[i + count + count] = get_rle_sprite(bmp3);

    m[i]                 = create_ppcol_mask(bmp1);
    m[i + count]         = create_ppcol_mask(bmp2);
    m[i + count + count] = create_ppcol_mask(bmp3);

		//MYCODE
		b[i]=bmp1;
  }

	
	//MYCOMMENT
  //destroy_bitmap(bmp1);
  destroy_bitmap(bmp2);
  destroy_bitmap(bmp3);
}

SpaceSprite::~SpaceSprite()
{
  int i;

  #ifdef SCP_DEBUG
  fprintf(debug_file, "? SpaceSprite -> Destructor\n");
  fflush(debug_file);
  #endif
  
  for(i = 0; i < (count * 3); i++) {
    destroy_ppcol_mask(m[i]);
    destroy_rle_sprite(s[i]);
  }

  #ifdef SCP_DEBUG
  fprintf(debug_file, "? SpaceSprite -> Destroyed successfully\n");
  fflush(debug_file);
  #endif
	//MYCODE
  for(i = 0; i < count; i++) {
	  destroy_bitmap(b[i]);
  }
}

 RLE_SPRITE *SpaceSprite::get_sprite(int index, int step)
{
  return(s[index + (step * count)]);
}

 BITMAP *SpaceSprite::get_bitmap(int index)
{
  return(b[index]);
}

void SpaceSprite::animate(int x, int y, int index, Frame *space)
{
  int spr_w = int(double(w) * space_zoom);
  int spr_h = int(double(h) * space_zoom);
  
  if(spr_w < 1) spr_w = 1;
  if(spr_h < 1) spr_h = 1;


	if (zoom_discrete)	
	  draw_rle_sprite(space->frame, s[index + (space_step * count)], x, y);
	else if (zoom_antia)
	  aa_stretch_sprite(space->frame, b[index], x, y, spr_w, spr_h);
	  //stretch_sprite(space->frame, b[index], x, y, spr_w, spr_h);
	else
	  stretch_sprite(space->frame, b[index], x, y, spr_w, spr_h);
  
  space->add_box(x, y, spr_w, spr_h);
}

void SpaceSprite::draw_character(int x, int y, int index, int color,
  BITMAP *bmp)
{

  // This routine had to be modified - bummer (crashes)
#ifdef SLOW_CHARACTER
  // Very slow and ugly way to do things, but for some reason,
  // using mask->sp_mask[row][chunk] crashes the bastard.

//orz: MSVC 98 (6.0?) gives me a warning about the method used for 
//addressing sp_mask.  does your version give you that warning?  
//do you suppose that that's the cause?  

  int i,j;
  for(i = 0; i < mask->h; i++)
    for(j = 0; j < mask->w; j++)
      if( check_ppmask_collision(mask, pix, 0, 0, i, j) )
        putpixel(bmp, x + i, y + j, color);
#else
  PPCOL_MASK *mask = m[index];
  int         row, chunk, pos, cx;
  for(row = 0; row < mask->h; row++) {
    cx = x;
    for(chunk = 0; chunk <= mask->max_chunk; chunk++) {
      int c = mask->sp_mask[row][chunk];
      for(pos = 0; pos < 32; pos+=1) {
        if (c & 0x80000000) putpixel(bmp, cx + pos, y + row, color);
        c = c << 1;
        }
      cx += 32;
      }
    }
#endif
}

//MYCODE
void SpaceSprite::draw_character_stretch(int x, int y, int index, int color,double zoom,
  BITMAP *bmp)
{
  int i, j; 
  PPCOL_MASK *mask = m[index];

 /* for(row = 0; row < mask->h; row++) 
      for(chunk = 0; chunk <= mask->max_chunk; chunk++) 
        for(pos = 0; (pos < 32) && ((pos + (chunk * 32)) < mask->w); pos++) 
          if( m[index]->sp_mask[row][chunk] & (0x80000000 >> (pos)) != 0) 
            putpixel(bmp, x + pos + (chunk * 32), y + row, color);

  // Very slow and ugly way to do things, but for some reason,
  // using mask->sp_mask[row][chunk] crashes the bastard.
	
  for(i = 0; i < mask->h; i++)
    for(j = 0; j < mask->w; j++)
      if( check_ppmask_collision(mask, pixel_mask, 0, 0, i, j) ) 
        putpixel(bmp, x + (int) zoom * i, y + (int) zoom * j, color);
				

	destroy_bitmap(temp_bmp);
	destroy_ppcol_mask(pixel_mask);*/
  for(i = 0; i < mask->h; i++)
    for(j = 0; j < mask->w; j++)
      if( check_ppmask_collision(mask, pix, 0, 0, i, j) ) 
        putpixel(bmp, x + int(i * zoom), y + int(j * zoom), color);

               
//        if((mask->sp_mask[row][chunk] & (0x80000000 >> pos)) != 0)
//          putpixel(bmp, x + int(zoom* (pos +  (chunk * 32))), y+int(zoom*row), color);
}

//MYCODE
void SpaceSprite::animate_character(int x, int y, int index, int color,
  Frame *space)
{
  int spr_w = int( double(w) * space_zoom)+2;
  int spr_h = int( double(h) * space_zoom)+2;

  if(spr_w < 1) spr_w = 1;
  if(spr_h < 1) spr_h = 1;

  #ifdef SCP_DEBUG
  fprintf(debug_file, "? SpaceSprite -> animate_character\n");
  fflush(debug_file);
  #endif

  if (zoom_discrete)
    draw_character(x, y, index + (space_step * count), color, space->frame);
  else
    draw_character_stretch(x,y,index,color,space_zoom,space->frame);

  space->add_box(x, y, spr_w, spr_h);
}

 int SpaceSprite::collide(int x, int y, int i, int ox, int oy, int oi,
  SpaceSprite *other)
{
  return(check_ppmask_collision(m[i], other->m[oi], x, y, ox, oy));
}

int line_collide;
int rect_x;
int rect_y;
int rect_w;
int rect_h;
int collide_x;
int collide_y;

PPCOL_MASK *rect_mask;

void check_line_collision(BITMAP *bmp, int x, int y, int d)
{
  int row, col;

  if(line_collide)
    return;

  col = rect_x - x;
  row = rect_y - y;

  // Oy...  Again wit de sp_mask problem...
	// Ain't good, really...  Oh well, this works, albeit slower...
	
/*	if((col >= 0) && (col < rect_w) &&
  if((col >= 0) && (col < rect_w) &&
     (row >= 0) && (row < rect_h)) {
    if( (rect_mask->sp_mask[row][col / 32] & (0x80000000 >> (col % 32))) != 0) {
    if((rect_mask->sp_mask[row][col / 32] & (0x80000000 >> (col % 32))) != 0) {
      line_collide = TRUE;
      collide_x = x;
      collide_y = y;
    }
  }*/

  if((col >= 0) && (col < rect_w) && (row >= 0) && (row < rect_h)) 
    if(check_ppmask_collision(rect_mask, pix, 0, 0, col, row)) {
      line_collide = TRUE;
      collide_x = x;
      collide_y = y;
    }
}

int SpaceSprite::collide_line(int lx1, int ly1, int *lx2, int *ly2,
  int sx, int sy, int sindex)
{
	line_collide = FALSE;
  rect_x = sx + (w / 2);
  rect_y = sy + (h / 2);
  rect_w = w;
  rect_h = h;
  rect_mask = m[sindex];
  do_line(NULL, lx1, ly1, *lx2, *ly2, 0, check_line_collision);
  if(line_collide) {
    *lx2 = collide_x;
    *ly2 = collide_y;
    return(TRUE);
  }

  return(FALSE);
}

 int SpaceSprite::frames()
{
  return(count);
}

 double SpaceSprite::width()
{
  return((double)(w));
}

 double SpaceSprite::height()
{
  return((double)(h));
}

SpaceRotatingSprite::SpaceRotatingSprite(DATAFILE *sprites, int sprite_count) :
  SpaceSprite(*this)
{
  int i, j, w2, h2, w3, h3;
  BITMAP *bmp, *bmp1, *bmp2, *bmp3;

  count = sprite_count;

  s = new RLE_SPRITE*[count * 3 * 64];
  m = new PPCOL_MASK*[count * 3 * 64];
	//MYCODE
  b = new BITMAP*[count * 64];

  w = ((RLE_SPRITE *)sprites[0].dat)->w;
  h = ((RLE_SPRITE *)sprites[0].dat)->h;

  w2 = w / 2; if(w2 < 1) w2 = 1;
  h2 = h / 2; if(h2 < 1) h2 = 1;
  w3 = w / 4; if(w3 < 1) w3 = 1;
  h3 = h / 4; if(h3 < 1) h3 = 1;

  bmp  = create_bitmap(w, h);
  bmp2 = create_bitmap(w2, h2);
  bmp3 = create_bitmap(w3, h3);

  for(i = 0; i < count; i++) {
    for(j = 0; j < 64; j++) {

			bmp1 = create_bitmap(w, h);

      clear_to_color(bmp, bitmap_mask_color(bmp));
      clear_to_color(bmp1, bitmap_mask_color(bmp1));
      clear_to_color(bmp2, bitmap_mask_color(bmp2));
      clear_to_color(bmp3, bitmap_mask_color(bmp3));

      tw_draw_rle_sprite(bmp, (RLE_SPRITE *)(sprites[i].dat), 0, 0);
      rotate_sprite(bmp1, bmp, 0, 0, itofix(4 * j));
      aa_stretch_sprite(bmp2, bmp1, 0, 0, w2, h2);
      aa_stretch_sprite(bmp3, bmp1, 0, 0, w3, h3);

      s[(i * 64) + j]                 = get_rle_sprite(bmp1);
      s[(i * 64) + j + (count * 64)]  = get_rle_sprite(bmp2);
      s[(i * 64) + j + (count * 128)] = get_rle_sprite(bmp3);

      m[(i * 64) + j]                 = create_ppcol_mask(bmp1);
      m[(i * 64) + j + (count * 64)]  = create_ppcol_mask(bmp2);
      m[(i * 64) + j + (count * 128)] = create_ppcol_mask(bmp3);
			
			//MYCODE
			b[(i * 64) + j]=bmp1;
    }
  }

	
  count *= 64;

  destroy_bitmap(bmp);
	//MYCOMMENT
  //destroy_bitmap(bmp1);
  destroy_bitmap(bmp2);
  destroy_bitmap(bmp3);
}
