

#include "../melee.h"

class ArilouSkiff : public Ship {
  int    weaponColor;
  double weaponRange;
  int    weaponFrames;
  int    weaponDamage;

  SpaceSprite *specialSprite;
  int          specialFrames;

  public:
  ArilouSkiff(double ox, double oy, double shipAngle, int shipCollideFlag,
    int shipRadarColor, ShipData *shipData);

  protected:
  int isBad_Warp(SpaceObject *o);
  virtual int activate_weapon();
  virtual int activate_special();
  virtual void calculate();
  virtual void calculate_gravity();
  virtual void pull(double angle, double pullVelocity);
};

class ArilouLaser : public SpaceLine {
  int frame;
  int frame_count;

  SpaceLocation *pos;
  double d;

  public:
  ArilouLaser(double langle, int lcolor, double lrange, int ldamage,
    int lfcount, SpaceLocation *opos, double oposdist);

  void calculate();
};

ArilouSkiff::ArilouSkiff(double ox, double oy, double shipAngle,
  int shipCollideFlag, int shipRadarColor, ShipData *shipData) :
  Ship(ox, oy, shipAngle, shipCollideFlag, shipRadarColor, shipData)
{
  specialSprite = data->spriteSpecial;

  weaponColor  = get_config_int("Weapon", "Color", 0);
  weaponRange  = get_config_float("Weapon", "Range", 0) * RANGE_RATIO;
  weaponFrames = get_config_int("Weapon", "Frames", 0);
  weaponDamage = get_config_int("Weapon", "Damage", 0);

  specialFrames = get_config_int("Special", "Frames", 0);
}

int ArilouSkiff::activate_weapon()
{
  int i;
  SpaceObject *o;
  SpaceLocation *l;

  l = NULL;
  double r = weaponRange;

  if((target) && (!target->isAlly(this))) {
    l = target;
    r = distance(l);
  }

  if (r >= weaponRange) {
   r = weaponRange;
    for(i = LAYER_SHIPS; (i <= LAYER_SHOTS); i++) {
      o = spaceFrame->getFirstObject(i);
      while (o != NULL) {
        if((distance(o) < r) && (!isAlly(o)) && (o->collideFlag() >= 0)) {
          l = o;
          r = distance(l);
          }
        o = o->next;
      }
    }

  }

  if (l) r = trajectory_angle(l); else r = angle;
   spaceFrame->addSpaceLine(new ArilouLaser(r, pallete_color[weaponColor], 
     weaponRange, weaponDamage, weaponFrames, this, w / 2.0));
  return TRUE;
}

int ArilouSkiff::activate_special()
{
  x = (double)(random() % (int)(X_MAX));
  y = (double)(random() % (int)(Y_MAX));

  if (isBad_Warp((SpaceObject*)spacePlanet))
  {
    crew  = 0;
    x = spacePlanet->normal_x();
    y = spacePlanet->normal_y();
    handle_damage();
   return(true);
  } else 
  if (target)
   if (isBad_Warp((SpaceObject*)target))
    {
      x=target->normal_x();    
      y=target->normal_y();    

      if (w > target->width()) 
       x-=w;
      else
       x-=target->width();      

      if (h > target->height()) 
       y-=h;
      else
       y-=target->height();
    }

   spaceFrame->addSpaceObject(LAYER_EXPLOSIONS, new Animation(x, y,
     specialSprite, specialFrames, 4));
  return(TRUE);
}

void ArilouSkiff::calculate()
{
  Ship::calculate();

  if(!thrust) {
    vx = 0.0;
    vy = 0.0;
    driftvx *= 0.9;
    driftvy *= 0.9;
  }
}

void ArilouSkiff::calculate_gravity()
{
}

void ArilouSkiff::pull(double angle, double pullVelocity)
{
}

ArilouLaser::ArilouLaser(double langle, int lcolor, double lrange,
  int ldamage, int lfcount, SpaceLocation *opos, double oposdist) :
  SpaceLine(0.0, 0.0, langle, opos->collideFlag(), lrange, lcolor),
  frame(0),
  frame_count(lfcount),
  pos(opos),
  d(oposdist)
{
  damage_factor = ldamage;
  x = pos->normal_x() + (d * cos(angle * ANGLE_RATIO));
  y = pos->normal_y() + (d * sin(angle * ANGLE_RATIO));

  if(!pos->exists())
    state = 0;
}

void ArilouLaser::calculate()
{
  if((frame < frame_count) && (pos->exists())) {
    x = pos->normal_x() + (d * cos(angle * ANGLE_RATIO));
    y = pos->normal_y() + (d * sin(angle * ANGLE_RATIO));
    SpaceLine::calculate();
    frame++;
  } else
    state = 0;
}

Ship *getArilouSkiff(double sX, double sY, double sAngle,
  int sCollideFlag, int sRadarColor, ShipData *sData)
{
  return(new ArilouSkiff(
    sX, sY, sAngle, sCollideFlag, sRadarColor, sData));
}

int ArilouSkiff::isBad_Warp(SpaceObject *o)
{
 double _d = distance_from(o->normal_x(), o->normal_y(), 
                              normal_x(), normal_y());

  if (_d <= w * 2)  //too dirty, but enough for planet
   return (TRUE);

 return (FALSE);
}

SHIP_REGISTER_INFO ArilouSkiffInfo =
{
  "arisk",
  "Arilou Skiff",
  getArilouSkiff
};
