
#include "../melee.h"

class EarthlingCruiser : public Ship {
  double       weaponRange;
  double       weaponVelocity;
  int          weaponDamage;
  int          weaponArmour;
  double       weaponTurnRate;

  int    specialColor;
  double specialRange;
  int    specialFrames;
  int    specialDamage;

  public:
  EarthlingCruiser(double ox, double oy, double shipAngle,
    int shipCollideFlag, int shipRadarColor, ShipData *shipData);

  protected:
  virtual int activate_weapon();
  virtual int activate_special();
};

class EarthlingMissile : public HomingMissile {
  public:
  EarthlingMissile(double ox, double oy, double oangle, double ov,
    int odamage, double orange, int oarmour, double otrate, Ship *oship,
    SpaceSprite *osprite);
};

EarthlingCruiser::EarthlingCruiser(double ox, double oy, double shipAngle,
  int shipCollideFlag, int shipRadarColor, ShipData *shipData) :
  Ship(ox, oy, shipAngle, shipCollideFlag, shipRadarColor, shipData) {

  weaponRange    = get_config_float("Weapon", "Range", 0) * RANGE_RATIO;
  weaponVelocity = get_config_float("Weapon", "Velocity", 0) / VELOCITY_RATIO;
  weaponDamage   = get_config_int("Weapon", "Damage", 0);
  weaponArmour   = get_config_int("Weapon", "Armour", 0);
  weaponTurnRate = get_config_float("Weapon", "TurnRate", 0);

  specialColor  = get_config_int("Special", "Color", 0);
  specialRange  = get_config_float("Special", "Range", 0) * RANGE_RATIO;
  specialFrames = get_config_int("Special", "Frames", 0);
  specialDamage = get_config_int("Special", "Damage", 0);
}

int EarthlingCruiser::activate_weapon() {
  spaceFrame->addSpaceObject(LAYER_SHOTS, new EarthlingMissile(
    0.0, -(height() / 2.0), angle, weaponVelocity, weaponDamage, weaponRange,
    weaponArmour, weaponTurnRate, this, data->spriteWeapon));
  return(TRUE);
}

int EarthlingCruiser::activate_special() {
  int fire = FALSE, i;
  SpaceObject *o;
  SpaceLocation *l = new SpaceLocation(normal_x(),
    normal_y() + (sin(angle * ANGLE_RATIO) * 10.0), 0.0, 0.0);

  for(i = 0; (i < SPACE_LAYERS) && (!fire); i++) {
    o = spaceFrame->getFirstObject(i);
    while(o != NULL) {
      if((!o->isAlly(this)) && (o->collideFlag() >= 0) &&
        (l->distance(o) < specialRange)) {
        spaceFrame->addSpaceLine(new Laser(trajectory_angle(o),
          pallete_color[specialColor], specialRange, 0,
          specialFrames, this, 0.0, 10.0));
        o->damage += specialDamage;
        o->handle_damage();
        play_sample((SAMPLE *)(melee[MELEE_BOOM + 0].dat), 255, 128, 1000, FALSE);
        fire = TRUE;
      }
      o = o->next;
    }
  }

  delete l;

  return(fire);
}

EarthlingMissile::EarthlingMissile(double ox, double oy, double oangle,
  double ov, int odamage, double orange, int oarmour, double otrate,
  Ship *oship, SpaceSprite *osprite) :
  HomingMissile(ox, oy, oangle, ov, odamage, orange, oarmour, otrate, oship,
    osprite)
{
  explosionSprite     = ship->data->spriteWeaponExplosion;
  explosionFrameCount = 10;
  explosionFrameSize  = 5;
}

Ship *getEarthlingCruiser(double sX, double sY, double sAngle,
  int sCollideFlag, int sRadarColor, ShipData *sData)
{
  return(new EarthlingCruiser(
    sX, sY, sAngle, sCollideFlag, sRadarColor, sData));
}

SHIP_REGISTER_INFO EarthlingCruiserInfo =
{
  "earcr",
  "Earthling Cruiser",
  getEarthlingCruiser
};
