
#include "../melee.h"

class MelnormeTrader : public Ship {
  double       weaponRange;
  double       weaponVelocity;
  int          weaponDamage;
  int          weaponArmour;
  SpaceObject *weaponObject;

  double       specialRange;
  double       specialVelocity;
  int          specialFrames;
  int          specialArmour;

  public:
  MelnormeTrader(double ox, double oy, double shipAngle, int shipCollideFlag,
    int shipRadarColor, ShipData *shipData);

  virtual void calculate();

  virtual int activate_weapon();
  virtual int activate_special();
};

class MelnormeShot : public Shot {
  double v;

  int frame;
  int frame_step;
  int frame_size;
  int frame_count;
  int charge_frame;
  int charge_phase;
  int released;

  public:
  MelnormeShot(double ox, double oy, double oangle, double ov, int odamage,
    double orange, int oarmour, Ship *oship, SpaceSprite *osprite,
    int ofcount, int ofsize);

  virtual void calculate();
  virtual void animateExplosion();
  virtual void handle_damage();
};

class MelnormeShotExplosion : public Animation {
  public:
  MelnormeShotExplosion(double ox, double oy, SpaceSprite *osprite,
    int last_frame, int frame_ticks, int first_frame);
};

class MelnormeDisable : public SpaceObject {
  Ship *ship;
  int   disableframe;
  int   disableframe_count;
  int   frame_step;
  int   frame_size;
  int   frame_count;

  public:
  MelnormeDisable(Ship *oship, SpaceSprite *osprite, int ofcount,
    int ofsize, int disableFrames);

  virtual void calculate();
};

class MelnormeSpecial : public AnimatedShot {
  int disableFrames;
  SpaceSprite *disableSprite;

  public:
  MelnormeSpecial(double ox, double oy, double oangle, double ov,
    int oframes, double orange, int oarmour, Ship *oship,
    SpaceSprite *osprite, int ofsize, int ofcount);

  int canCollide(SpaceLocation *other) ;
  void inflict_damage(SpaceObject *other);
  void animateExplosion();
};

MelnormeTrader::MelnormeTrader(double ox, double oy, double shipAngle,
  int shipCollideFlag, int shipRadarColor, ShipData *shipData) :
  Ship(ox, oy, shipAngle, shipCollideFlag, shipRadarColor, shipData)
{

  weaponRange    = get_config_float("Weapon", "Range", 0) * RANGE_RATIO;
  weaponVelocity = get_config_float("Weapon", "Velocity", 0) / VELOCITY_RATIO;
  weaponDamage   = get_config_int("Weapon", "Damage", 0);
  weaponArmour   = get_config_int("Weapon", "Armour", 0);
  weaponObject   = NULL;

  specialRange    = get_config_float("Special", "Range", 0) * RANGE_RATIO;
  specialVelocity = get_config_float("Special", "Velocity", 0) / VELOCITY_RATIO;
  specialFrames   = get_config_int("Special", "Frames", 0);
  specialArmour   = get_config_int("Special", "Armour", 0);
}

void MelnormeTrader::calculate()
{
  if (weaponObject) {
    if (!weaponObject->exists()) weaponObject = NULL;
    if (fire_weapon) recharge_step -= 1; else weaponObject = NULL;
    }
  Ship::calculate();
}

int MelnormeTrader::activate_weapon()
{
  if(weaponObject)
    return(FALSE);
  weaponObject = spaceFrame->addSpaceObject(LAYER_SHOTS, new MelnormeShot(
    0.0, -(height() / 2.0), angle, weaponVelocity, weaponDamage, weaponRange,
    weaponArmour, this, data->spriteWeapon, 10, 5));
  return(TRUE);
}

int MelnormeTrader::activate_special() {
  spaceFrame->addSpaceObject(LAYER_SHOTS, new MelnormeSpecial(
    0.0, -(height() / 2.0), angle, specialVelocity, specialFrames,
    specialRange, specialArmour, this, data->spriteSpecial, 20, 5));
  return(TRUE);
}

MelnormeShot::MelnormeShot(double ox, double oy, double oangle, double ov,
  int odamage, double orange, int oarmour, Ship *oship, SpaceSprite *osprite,
  int ofcount, int ofsize) :
  Shot(ox, oy, oangle, ov, odamage, orange, oarmour, oship, oship, osprite),
  v(ov),
  frame(0),
  frame_step(0),
  frame_size(ofsize),
  frame_count(ofcount),
  charge_frame(0),
  charge_phase(0),
  released(FALSE)
{
  vx = 0.0;
  vy = 0.0;
}

void MelnormeShot::calculate()
{
  if(released)
    Shot::calculate();
  else {
    SpaceObject::calculate();
    if(!ship->exists()) {
      state = 0;
      return;
    }
  }

  frame_step++;
  if(frame_step == frame_size) {
    frame_step = 0;
    frame++;
    if(frame == frame_count) {
      frame = 0;
      if((!released) && (charge_phase < 3)) {
        charge_frame++;
        if(charge_frame == 5) {
          charge_frame = 0;
          charge_phase++;
          damage_factor *= 2;
          armour *= 2;
          play_sample(ship->data->sampleWeapon[0], 255, 128, 1000, FALSE);
        }
      }
    }
  }

  sprite_index = (charge_phase * 10) + frame;
  if(!released) {
    angle = ship->get_angle();
    x = ship->normal_x() + (cos(angle * ANGLE_RATIO) * (ship->width() / 2.0));
    y = ship->normal_y() + (sin(angle * ANGLE_RATIO) * (ship->height() / 2.0));
  }
  if((!released) && (!ship->fire_weapon)) {
    vx = cos(angle * ANGLE_RATIO) * v;
    vy = sin(angle * ANGLE_RATIO) * v;
    released = TRUE;
    play_sample(ship->data->sampleSpecial[0], 255, 128, 1000, FALSE);
  }
}

void MelnormeShot::animateExplosion()
{
  spaceFrame->addSpaceObject(LAYER_EXPLOSIONS, new MelnormeShotExplosion(x, y,
    ship->data->spriteWeaponExplosion, (charge_phase * 20) + 20, 5,
    charge_phase * 20));
}

void MelnormeShot::handle_damage()
{
  int old = armour;
  Shot::handle_damage();
  if (!released && (armour > 0)) armour = old;
}
    
MelnormeShotExplosion::MelnormeShotExplosion(double ox, double oy,
  SpaceSprite *osprite, int last_frame, int frame_ticks, int first_frame) :
  Animation(ox, oy, osprite, last_frame, frame_ticks) {
  sprite_index = first_frame;
}

MelnormeDisable::MelnormeDisable(Ship *oship, SpaceSprite *osprite,
  int ofcount, int ofsize, int disableFrames) :
  SpaceObject(oship->normal_x(), oship->normal_y(), 0.0, -1, osprite),
  ship(oship),
  disableframe(0),
  disableframe_count(disableFrames),
  frame_step(0),
  frame_size(ofsize),
  frame_count(ofcount)
{
}

void MelnormeDisable::calculate()
{
  frame_step++;
  if(frame_step == frame_size) {
    frame_step = 0;
    sprite_index++;
    if(sprite_index == frame_count)
      sprite_index = 0;
  }
  if(!ship->exists()) {
    state = 0;
    return;
  }
  x = ship->normal_x();
  y = ship->normal_y();
  ship->nextkeys &= ~(KEYFLAG_LEFT | KEYFLAG_RIGHT | KEYFLAG_SPECIAL);
  ship->turn_step += 0.4;
  disableframe++;
  if(disableframe == disableframe_count)
    state = 0;
  SpaceObject::calculate();
}

MelnormeSpecial::MelnormeSpecial(double ox, double oy, double oangle,
  double ov, int oframes, double orange, int oarmour, Ship *oship,
  SpaceSprite *osprite, int ofcount, int ofsize) :
  AnimatedShot(ox, oy, oangle, ov, 0, orange, oarmour, oship, oship, osprite,
    ofcount, ofsize),
  disableFrames(oframes),
  disableSprite(oship->data->spriteExtra)
{
}

void MelnormeSpecial::animateExplosion() {}

int MelnormeSpecial::canCollide(SpaceLocation *other) {
  if (other->getID() != SPACE_SHIP) return FALSE;
  return TRUE;
}

void MelnormeSpecial::inflict_damage(SpaceObject *other) {
  if(other->getID() == SPACE_SHIP)
    spaceFrame->addSpaceObject(LAYER_EXPLOSIONS, new MelnormeDisable(
      (Ship *)(other), disableSprite, 20, 5, disableFrames));
  state = 0;
}

Ship *getMelnormeTrader(double sX, double sY, double sAngle,
  int sCollideFlag, int sRadarColor, ShipData *sData)
{
  return(new MelnormeTrader(
    sX, sY, sAngle, sCollideFlag, sRadarColor, sData));
}

SHIP_REGISTER_INFO MelnormeTraderInfo =
{
  "meltr",
  "Melnorme Trader",
  getMelnormeTrader
};
