
#include "../melee.h"

class MyconPodship : public Ship {
  double       weaponRange;
  double       weaponVelocity;
  int          weaponDamage;
  double       weaponHome;
  int          specialRepair;

  public:
  MyconPodship(double ox, double oy, double shipAngle, int shipCollideFlag,
    int shipRadarColor, ShipData *shipData);

  virtual int activate_weapon();
  virtual int activate_special();
};

class MyconPlasma : public HomingMissile {
  double v;

  int frame_count;
  int max_damage;

  public:
  MyconPlasma(double ox, double oy, double oangle, double ov, int odamage,
    double orange, double otrate, Ship *oship, SpaceSprite *osprite, int ofcount);

  virtual void calculate();

  virtual void inflict_damage(SpaceObject *other);
  virtual void handle_damage();
  virtual int canCollide(SpaceLocation *other);
};

int MyconPlasma::canCollide(SpaceLocation *other) {
  if (isAlly(other) && (other->getID() == getID())) return 0;
  return HomingMissile::canCollide(other);
}

MyconPodship::MyconPodship(double ox, double oy, double shipAngle,
  int shipCollideFlag, int shipRadarColor, ShipData *shipData) :
  Ship(ox, oy, shipAngle, shipCollideFlag, shipRadarColor, shipData)
{

  weaponRange    = get_config_float("Weapon", "Range", 0) * RANGE_RATIO;
  weaponVelocity = get_config_float("Weapon", "Velocity", 0) / VELOCITY_RATIO;
  weaponDamage   = get_config_int("Weapon", "Damage", 0);
  weaponHome     = get_config_int("Weapon", "Homing", 0);
  specialRepair  = get_config_int("Special", "Repair", 0);
}

int MyconPodship::activate_weapon()
{
  spaceFrame->addSpaceObject(LAYER_SHOTS, new MyconPlasma(0.0, -height(),
    angle, weaponVelocity, weaponDamage, weaponRange, weaponHome, this,
    data->spriteWeapon, 64));
  return(TRUE);
}

int MyconPodship::activate_special()
{
  if(crew >= crew_max)
    return(FALSE);
  repair += specialRepair;
  handle_damage();
  return(TRUE);
}

MyconPlasma::MyconPlasma(double ox, double oy, double oangle, double ov,
    int odamage, double orange, double otrate, Ship *oship,
    SpaceSprite *osprite, int ofcount) :
  HomingMissile(ox, oy, oangle, ov, odamage, orange, 0, otrate, oship,
      osprite),
  v(ov),
  frame_count(ofcount),
  max_damage(odamage)
{
  collide_flag = 0;
}

void MyconPlasma::calculate()
{
  HomingMissile::calculate();

  sprite_index = (int)((d / range) * (double)(frame_count - 1));
  damage_factor = max_damage - (int)((d / range) * (double)(max_damage));
}

void MyconPlasma::inflict_damage(SpaceObject *other)
{
  SpaceObject::inflict_damage(other);
  if(other->getID() != SPACE_SHOT) {
    spaceFrame->addSpaceObject(LAYER_EXPLOSIONS, new FixedAnimation(other,
      ship->data->spriteWeaponExplosion, 20, 5));
    state = 0;
  }
}

void MyconPlasma::handle_damage() {
  int total_damage = 0;

  if((damage > 0) || (directDamage > 0)) {
    total_damage += damage;
    total_damage += directDamage;
    d += ((double)(total_damage) / (double)(max_damage) * range);
    damage = 0;
    directDamage = 0;
    if(d >= range)
      state = 0;
  }
}

Ship *getMyconPodship(double sX, double sY, double sAngle,
  int sCollideFlag, int sRadarColor, ShipData *sData)
{
  return(new MyconPodship(
    sX, sY, sAngle, sCollideFlag, sRadarColor, sData));
}

SHIP_REGISTER_INFO MyconPodshipInfo =
{
  "mycpo",
  "Mycon Podship",
  getMyconPodship
};
