
#include "../melee.h"

class SlylandroProbe : public Ship {
  int frame;
  int thrustActive;
  int thrustForward;

  public:
  SlylandroProbe(double ox, double oy, double shipAngle, int shipCollideFlag,
    int shipRadarColor, ShipData *shipData);

  int crewPanelColor();

  virtual void calculate_thrust();
  virtual int activate_weapon();
  virtual void calculate_fire_special();
  virtual void calculate();
  virtual int isCrewRobots();
};

class SlylandroLaser : public Laser {
  int oldDamageFactor;

  public:
  SlylandroLaser(int ldamage, int lfcount, SpaceLocation *opos);

  virtual void calculate();
};

SlylandroProbe::SlylandroProbe(double ox, double oy, double shipAngle,
  int shipCollideFlag, int shipRadarColor, ShipData *shipData) :
  Ship(ox, oy, shipAngle, shipCollideFlag, shipRadarColor, shipData),
  frame(0),
  thrustActive(FALSE),
  thrustForward(TRUE)
{
}

int SlylandroProbe::crewPanelColor()
{
  return pallete_color[8];
}

void SlylandroProbe::calculate()
{
  if(phase_count < PHASE_MAX) {
    calculate_phase();
    return;
  }

  if(batt < batt_max) {
    recharge_step++;
    if(recharge_step == recharge_rate) {
      batt += recharge_amount;
      recharge_step = 0;
    }
  }

  if(weapon_recharge > 0)
    weapon_recharge--;
  if(special_recharge > 0)
    special_recharge--;

  calculate_thrust();
  calculate_turn_left();
  calculate_turn_right();
  calculate_fire_weapon();
  calculate_fire_special();

  while(fabs(turn_step) > 5.625) {
    if(turn_step < 0.0) {
      angle -= 5.625;
      turn_step += 5.625;
    } else
    if(turn_step > 0.0) {
      angle += 5.625;
      turn_step -= 5.625;
    }

    if(angle < 0.0)
      angle += 360.0;
    if(angle >= 360.0)
      angle -= 360.0;
  }

  frame++;
  if(frame == 5) {
    frame = 0;
    if(thrustForward) {
      sprite_index++;
      if(sprite_index == 64)
        sprite_index = 0;
    } else {
      sprite_index--;
      if(sprite_index == -1)
        sprite_index = 63;
    }
  }

  SpaceObject::calculate();
}

void SlylandroProbe::calculate_thrust()
{
  double v;

  vx += (cos(angle * ANGLE_RATIO) * accel_rate);
  vy += (sin(angle * ANGLE_RATIO) * accel_rate);

  v = sqrt((vx * vx) + (vy * vy));

  if(v > speed_max) {
    vx = vx * speed_max / v;
    vy = vy * speed_max / v;
  }

  driftvx *= 0.99;
  driftvy *= 0.99;

  if(!thrust)
    thrustActive = FALSE;
  else
  if(!thrustActive) {
    thrustActive = TRUE;
    thrustForward = !thrustForward;
    angle = angle + 180.0;
    if(angle > 360.0)
      angle -= 360.0;
  }
}

int SlylandroProbe::activate_weapon()
{
  SlylandroLaser *laser[5];

  laser[0] = new SlylandroLaser(0, 80, this);
  laser[1] = new SlylandroLaser(0, 80, laser[0]);
  laser[2] = new SlylandroLaser(0, 80, laser[1]);
  laser[3] = new SlylandroLaser(0, 80, laser[2]);
  laser[4] = new SlylandroLaser(1, 80, laser[3]);
  spaceFrame->addSpaceLine(laser[4]);
  spaceFrame->addSpaceLine(laser[3]);
  spaceFrame->addSpaceLine(laser[2]);
  spaceFrame->addSpaceLine(laser[1]);
  spaceFrame->addSpaceLine(laser[0]);

  return(TRUE);
}

void SlylandroProbe::calculate_fire_special()
{
  int fire = FALSE, i;
  SpaceObject *o;

  special_low = FALSE;

  if(fire_special) {
    if(special_recharge != 0)
      return;

    for(i = 0; (i < SPACE_LAYERS) && (!fire); i++) {
      o = spaceFrame->getFirstObject(i);
      while((o != NULL) && (!fire)) {
        if((o->getID() == SPACE_ASTEROID) && (distance(o) < w)) {
          damage_factor = 1;
          inflict_damage(o);
          damage_factor = 0;
          batt = batt_max;
          fire = TRUE;
        }
        o = o->next;
      }
    }

    if(!fire)
      return;

    special_recharge = special_rate;

    stop_sample(data->sampleSpecial[0]);
    play_sample(data->sampleSpecial[0], 255, 128, 1000, FALSE);
  }
}

SlylandroLaser::SlylandroLaser(int ldamage, int lfcount, SpaceLocation *opos) :
  Laser(0.0, pallete_color[64], 0.0, ldamage, lfcount, opos, 0.0, 0.0),
  oldDamageFactor(ldamage)
{
  length = (double)(random() % (50 - abs(frame - 40)) * 2);
  if(pos->getID() == SPACE_SHIP) {
    x = pos->normal_x();
    y = pos->normal_y();
    if((((Ship *)(pos))->target) && (!((Ship *)(pos))->target->isAlly(pos)))
      angle = trajectory_angle(((Ship *)(pos))->target) + (double)(random() % 45) - 22.5;
    else
      angle = (double)(random() % 360);
  } else {
    x = pos->normal_x() + ((SlylandroLaser *)(pos))->edge_x();
    y = pos->normal_y() + ((SlylandroLaser *)(pos))->edge_y();
    angle = pos->get_angle() + (double)(random() % 45) - 22.5;
  }
}

void SlylandroLaser::calculate()
{
  if((frame < frame_count) && (pos->exists())) {
    if(pos->getID() == SPACE_SHIP) {
      x = pos->normal_x();
      y = pos->normal_y();
      if((frame % 5) == 0) {
        color  = pallete_color[64 + (frame / 5)];
        if((((Ship *)(pos))->target) && (!((Ship *)(pos))->target->isAlly(pos)))
          angle = trajectory_angle(((Ship *)(pos))->target) + (double)(random() % 90) - 45.0;
        else
          angle = (double)(random() % 360);
        length = (double)(random() % (50 - abs(frame - 40)) * 2);
        damage_factor = oldDamageFactor;
      }
    } else {
      if((frame % 5) == 0) {
        color  = pallete_color[64 + (frame / 5)];
        angle  = pos->get_angle() + (double)(random() % 90) - 45.0;
        length = (double)(random() % (50 - abs(frame - 40)) * 2);
        damage_factor = oldDamageFactor;
      }
      x = pos->normal_x() + ((SlylandroLaser *)(pos))->edge_x();
      y = pos->normal_y() + ((SlylandroLaser *)(pos))->edge_y();
    }
    SpaceLine::calculate();
    frame++;
  } else
    state = 0;
}

Ship *getSlylandroProbe(double sX, double sY, double sAngle,
  int sCollideFlag, int sRadarColor, ShipData *sData)
{
  return(new SlylandroProbe(
    sX, sY, sAngle, sCollideFlag, sRadarColor, sData));
}

int SlylandroProbe::isCrewRobots()
{
 return (TRUE);
}

SHIP_REGISTER_INFO SlylandroProbeInfo =
{
  "slypr",
  "Slylandro Probe",
  getSlylandroProbe
};
