
#include "../melee.h"

class SupoxBlade : public Ship {
  double       weaponRange;
  double       weaponVelocity;
  int          weaponDamage;
  int          weaponArmour;

  public:
  SupoxBlade(double ox, double oy, double shipAngle, int shipCollideFlag,
    int shipRadarColor, ShipData *shipData);

  virtual void calculate_thrust();
  virtual void calculate_turn_left();
  virtual void calculate_turn_right();
  virtual void calculate_hotspots();

  virtual int activate_weapon();
  virtual void calculate_fire_special();
};

SupoxBlade::SupoxBlade(double ox, double oy, double shipAngle,
  int shipCollideFlag, int shipRadarColor, ShipData *shipData) :
  Ship(ox, oy, shipAngle, shipCollideFlag, shipRadarColor, shipData)
{

  weaponRange    = get_config_float("Weapon", "Range", 0) * RANGE_RATIO;
  weaponVelocity = get_config_float("Weapon", "Velocity", 0) / VELOCITY_RATIO;
  weaponDamage   = get_config_int("Weapon", "Damage", 0);
  weaponArmour   = get_config_int("Weapon", "Armour", 0);
}

void SupoxBlade::calculate_thrust()
{
  double v;

  if(thrust || ((fire_special) && (turn_left || turn_right || thrust))) {
    if((fire_special) && (turn_left)) {
      vx += (cos((angle - 90.0) * ANGLE_RATIO) * accel_rate);
      vy += (sin((angle - 90.0) * ANGLE_RATIO) * accel_rate);
    }
    if((fire_special) && (turn_right)) {
      vx += (cos((angle + 90.0) * ANGLE_RATIO) * accel_rate);
      vy += (sin((angle + 90.0) * ANGLE_RATIO) * accel_rate);
    }
    if((fire_special) && (thrust)) {
      vx += (cos((angle + 180.0) * ANGLE_RATIO) * accel_rate);
      vy += (sin((angle + 180.0) * ANGLE_RATIO) * accel_rate);
    }
    if((!fire_special) && (thrust)) {
      vx += (cos(angle * ANGLE_RATIO) * accel_rate);
      vy += (sin(angle * ANGLE_RATIO) * accel_rate);
    }

    v = sqrt((vx * vx) + (vy * vy));

    if((v > speed_max) && (v != 0.0)) {
      vx = vx * speed_max / v;
      vy = vy * speed_max / v;
    }

    driftvx *= 0.99;
    driftvy *= 0.99;
  }
}

void SupoxBlade::calculate_turn_left()
{
  if(!fire_special)
    Ship::calculate_turn_left();
}

void SupoxBlade::calculate_turn_right()
{
  if(!fire_special)
    Ship::calculate_turn_right();
}

void SupoxBlade::calculate_hotspots()
{
  if(!fire_special)
    Ship::calculate_hotspots();
}

int SupoxBlade::activate_weapon()
{
  spaceFrame->addSpaceObject(LAYER_SHOTS, new Missile(0.0, -(height() / 2.0),
    angle, weaponVelocity, weaponDamage, weaponRange, weaponArmour,
    this, this, data->spriteWeapon));
  return(TRUE);
}

void SupoxBlade::calculate_fire_special() {}

Ship *getSupoxBlade(double sX, double sY, double sAngle,
  int sCollideFlag, int sRadarColor, ShipData *sData)
{
  return(new SupoxBlade(sX, sY, sAngle, sCollideFlag, sRadarColor, sData));
}

SHIP_REGISTER_INFO SupoxBladeInfo =
{
  "supbl",
  "Supox Blade",
  getSupoxBlade
};
