
#include "../melee.h"

class UmgahDrone : public Ship {
  int          weaponDamage;
  int          weaponCone;
  int          specialRate;

  public:
  UmgahDrone(double ox, double oy, double shipAngle,
    int shipCollideFlag, int shipRadarColor, ShipData *shipData);

  virtual int activate_weapon();
  virtual int activate_special();
  virtual void calculate_thrust();
  virtual void calculate();
};

class UmgahCone : public SpaceObject {
  double dist;
  Ship  *ship;

  int frame;
  int frame_count;
  int cone;

  public:
  UmgahCone(double odist, int odamage, Ship *oship, SpaceSprite *osprite,
    int num_frames, int cone_index);

  virtual void calculate();
  virtual void inflict_damage(SpaceObject *other);
};

UmgahDrone::UmgahDrone(double ox, double oy, double shipAngle,
  int shipCollideFlag, int shipRadarColor, ShipData *shipData) :
  Ship(ox, oy, shipAngle, shipCollideFlag, shipRadarColor, shipData)
{

  weaponDamage   = get_config_int("Weapon", "Damage", 0);
  weaponCone     = random() % 6;
  specialRate    = special_rate;
}

int UmgahDrone::activate_weapon()
{
  int i = random() % 6;

  spaceFrame->addSpaceObject(LAYER_SHOTS, new UmgahCone(
    height() / 2.0, weaponDamage, this, data->spriteWeapon, 5, weaponCone));
  while(i == weaponCone)
    i = random() % 6;
  weaponCone = i;
  return(TRUE);
}

int UmgahDrone::activate_special()
{
  special_rate = specialRate;
  vx = 0.0;
  vy = 0.0;
  driftvx = 0.0;
  driftvy = 0.0;
  x -= (cos(angle * ANGLE_RATIO) * width() * 2.0);
  y -= (sin(angle * ANGLE_RATIO) * height() * 2.0);
  if(!thrust)
    special_rate = 5;
  return(TRUE);
}

void UmgahDrone::calculate_thrust()
{
  if(special_recharge == 0)
    Ship::calculate_thrust();
}

void UmgahDrone::calculate()
{
  if (fire_weapon)
    recharge_step = 0;

 Ship::calculate();
}

UmgahCone::UmgahCone(double odist, int odamage, Ship *oship,
  SpaceSprite *osprite, int num_frames, int cone_index) :
  SpaceObject(oship->normal_x(), oship->normal_y(), 0.0,
    oship->collideFlag(), osprite),
  dist(odist),
  ship(oship),
  frame(0),
  frame_count(num_frames),
  cone(cone_index)
{
  id = SPACE_SHOT;
  damage_factor = odamage;
  x += cos(ship->get_angle() * ANGLE_RATIO) * dist;
  y += sin(ship->get_angle() * ANGLE_RATIO) * dist;
  sprite_index = (int)((ship->get_angle() + 90.0) / 5.625);
  while(sprite_index < 0)
    sprite_index += 64;
  while(sprite_index >= 64)
    sprite_index -= 64;
  sprite_index += (64 * cone);

  if(!ship->exists())
    state = 0;
}

void UmgahCone::calculate()
{
  if(!ship->exists()) {
    state = 0;
    return;
  }

  if(frame == frame_count)
    state = 0;
  frame++;
  x = ship->normal_x() + (cos(ship->get_angle() * ANGLE_RATIO) * dist);
  y = ship->normal_y() + (sin(ship->get_angle() * ANGLE_RATIO) * dist);
  sprite_index = (int)((ship->get_angle() + 90.0) / 5.625);
  while(sprite_index < 0)
    sprite_index += 64;
  while(sprite_index >= 64)
    sprite_index -= 64;
  sprite_index += (64 * cone);
  SpaceObject::calculate();
}

void UmgahCone::inflict_damage(SpaceObject *other)
{
  SpaceObject::inflict_damage(other); 
  damage_factor = 0;
}

Ship *getUmgahDrone(double sX, double sY, double sAngle,
  int sCollideFlag, int sRadarColor, ShipData *sData)
{
  return(new UmgahDrone(sX, sY, sAngle, sCollideFlag, sRadarColor, sData));
}

SHIP_REGISTER_INFO UmgahDroneInfo =
{
  "umgdr",
  "Umgah Drone",
  getUmgahDrone
};
