
#include <math.h>
#include "../melee.h"

class ZoqFotPikStinger : public Ship {
  double       weaponRange;
  double       weaponVelocity;
  int          weaponDamage;
  int          weaponArmour;

  int          specialDamage;

  public:
  ZoqFotPikStinger(double ox, double oy, double shipAngle,
    int shipCollideFlag, int shipRadarColor, ShipData *shipData);

  virtual int activate_weapon();
  virtual int activate_special();
};

class ZoqFotPikShot : public Shot {
  int frame_count;

  public:
  ZoqFotPikShot(double ox, double oy, double oangle, double ov, int odamage,
    double orange, int oarmour, Ship *oship, SpaceSprite *osprite,
    int num_frames);

  virtual void calculate();
};

class ZoqFotPikTongue : public SpaceObject {
  double dist;
  Ship  *ship;
  int    lick_factor;

  int frame;
  int frame_size;
  int frame_count;
  int frame_step;

  public:
  ZoqFotPikTongue(double odist, int odamage, Ship *oship,
    SpaceSprite *osprite, int ofcount, int ofsize);

  virtual void calculate();
  virtual void inflict_damage(SpaceObject *other);
};

ZoqFotPikStinger::ZoqFotPikStinger(double ox, double oy, double shipAngle,
  int shipCollideFlag, int shipRadarColor, ShipData *shipData) :
  Ship(ox, oy, shipAngle, shipCollideFlag, shipRadarColor, shipData)
{

  weaponRange    = get_config_float("Weapon", "Range", 0) * RANGE_RATIO;
  weaponVelocity = get_config_float("Weapon", "Velocity", 0) / VELOCITY_RATIO;
  weaponDamage   = get_config_int("Weapon", "Damage", 0);
  weaponArmour   = get_config_int("Weapon", "Armour", 0);

  specialDamage = get_config_int("Special", "Damage", 0);
}

int ZoqFotPikStinger::activate_weapon() {
  spaceFrame->addSpaceObject(LAYER_SHOTS, new ZoqFotPikShot(
    0.0, -(height() / 2.0), angle - 15.0 + ((double)(random() % 3) * 10.0),
    weaponVelocity, weaponDamage, weaponRange, weaponArmour, this,
    data->spriteWeapon, 6));
  return(TRUE);
}

int ZoqFotPikStinger::activate_special()
{
  spaceFrame->addSpaceObject(LAYER_HOTSPOTS, new ZoqFotPikTongue(
    height() / 2.0, specialDamage, this, data->spriteSpecial, 6, 5));
  return(TRUE);
}

ZoqFotPikShot::ZoqFotPikShot(double ox, double oy, double oangle,
  double ov, int odamage, double orange, int oarmour, Ship *oship,
  SpaceSprite *osprite, int num_frames) :
  Shot(ox, oy, oangle, ov, odamage, orange, oarmour, oship, oship, osprite),
  frame_count(num_frames)
{
}

void ZoqFotPikShot::calculate()
{
  Shot::calculate();
  sprite_index = (int)((d / range) * (double)(frame_count - 1));
}

ZoqFotPikTongue::ZoqFotPikTongue(double odist, int odamage, Ship *oship,
  SpaceSprite *osprite, int ofcount, int ofsize) :
  SpaceObject(oship->normal_x(), oship->normal_y(), 0.0,
    oship->collideFlag(), osprite),
  dist(odist),
  ship(oship),
  lick_factor(odamage),
  frame(0),
  frame_size(ofsize),
  frame_count(ofcount),
  frame_step(0)
{
  id = SPACE_SHOT;
  damage_factor = lick_factor;
  x += cos(ship->get_angle() * ANGLE_RATIO) * dist;
  y += sin(ship->get_angle() * ANGLE_RATIO) * dist;
  sprite_index = (int)((ship->get_angle() + 90.0) / 5.625);
  while(sprite_index < 0)
    sprite_index += 64;
  while(sprite_index >= 64)
    sprite_index -= 64;
  if(!ship->exists())
    state = 0;
}

void ZoqFotPikTongue::calculate()
{
  int current_frame = frame;

  if(!ship->exists()) {
    state = 0;
    return;
  }

  frame_step++;
  if(frame_step == frame_size) {
    frame_step = 0;
    frame++;
    if(frame == frame_count)
      state = 0;
  }
  x = ship->normal_x() + (cos(ship->get_angle() * ANGLE_RATIO) * dist);
  y = ship->normal_y() + (sin(ship->get_angle() * ANGLE_RATIO) * dist);
  if(current_frame != frame)
    damage_factor = lick_factor;
  sprite_index = (int)((ship->get_angle() + 90.0) / 5.625);
  while(sprite_index < 0)
    sprite_index += 64;
  while(sprite_index >= 64)
    sprite_index -= 64;
  sprite_index += (64 * frame);
  SpaceObject::calculate();
}

void ZoqFotPikTongue::inflict_damage(SpaceObject *other)
{
  SpaceObject::inflict_damage(other); 
  damage_factor = 0;
}

Ship *getZoqFotPikStinger(double sX, double sY, double sAngle,
  int sCollideFlag, int sRadarColor, ShipData *sData) {
  return(new ZoqFotPikStinger(
    sX, sY, sAngle, sCollideFlag, sRadarColor, sData));
}

SHIP_REGISTER_INFO ZoqFotPikStingerInfo =
{
  "zfpst",
  "Zoq-Fot-Pik Stinger",
  getZoqFotPikStinger
};
